import logging
import os
import re
import shutil
import tarfile
import textwrap
import subprocess
import tempfile

from pyrpkg.errors import rpkgError

log = logging.getLogger("__main__")

pack_sources_path = os.path.join(
    os.path.dirname(__file__), 'preproc', 'scripts', 'pack_sources.sh')


def pack_sources(dir_to_pack, target_path, pack_dir_as):
    """
    Create a gzipped tar archive from the given directory.

    :param str dir_to_pack: directory to be packed
    :param str target_path: path to the resulting archive
    :param str pack_dir_as: packed directory name inside the archive
    """
    log.info("Packing {0} as {1} into {2}...".format(
        dir_to_pack, pack_dir_as, target_path))

    tempdir = tempfile.mkdtemp()
    target_tmp_path = os.path.join(tempdir, os.path.basename(target_path))
    run_command([pack_sources_path, dir_to_pack, pack_dir_as, target_tmp_path])
    shutil.move(target_tmp_path, target_path)
    shutil.rmtree(tempdir)


def find_source_zero(rpm_sources):
    """
    For the given list of rpm_sources,
    return filename of the Source0.

    :param list rpm_sources

    :returns str: filename of Source0 or None
    """
    for (filepath, num, flags) in rpm_sources:
        if num == 0 and flags == 1:
            return os.path.basename(filepath)
    return None


def get_changelog_entry(commit_message):
    entry = '- ' + commit_message.split('\n')[0] if commit_message else ''
    return '\n'.join(textwrap.wrap(re.sub(r'(\s+)%([^%])', r'\1%%\2', entry), 80))


def run_command(cmd, shell=False, env=None, cwd=None):
    process = subprocess.Popen(
        cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
        shell=shell, env=env, cwd=cwd)
    (stdout, stderr) = process.communicate()
    if process.returncode != 0:
        raise rpkgError(stderr)
    return stdout.decode('utf-8')


def edit(editor, text):
    tmp_f = tempfile.NamedTemporaryFile('w', delete=False)
    tmp_f.write(text)
    tmp_f.close()
    subprocess.check_call(editor.split()+[tmp_f.name])
    tmp_f = open(tmp_f.name, 'r')
    result = tmp_f.read()
    tmp_f.close()
    os.unlink(tmp_f.name)
    return result
